@@ -93,8 +93,6 @@ enum
 /* the one instance of the systemclock */
 static GstClock *_the_system_clock = NULL;
 
-static void gst_system_clock_class_init (GstSystemClockClass * klass);
-static void gst_system_clock_init (GstSystemClock * clock);
 static void gst_system_clock_dispose (GObject * object);
 static void gst_system_clock_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
@@ -144,7 +142,8 @@ gst_system_clock_class_init (GstSystemClockClass * klass)
   g_object_class_install_property (gobject_class, PROP_CLOCK_TYPE,
       g_param_spec_enum ("clock-type", "Clock type",
           "The type of underlying clock implementation used",
-          GST_TYPE_CLOCK_TYPE, DEFAULT_CLOCK_TYPE, G_PARAM_READWRITE));
+          GST_TYPE_CLOCK_TYPE, DEFAULT_CLOCK_TYPE,
+          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 
   gstclock_class->get_internal_time = gst_system_clock_get_internal_time;
   gstclock_class->get_resolution = gst_system_clock_get_resolution;
@@ -200,8 +199,6 @@ gst_system_clock_dispose (GObject * object)
     GST_CAT_DEBUG (GST_CAT_CLOCK, "unscheduling entry %p", entry);
     entry->status = GST_CLOCK_UNSCHEDULED;
   }
-  g_list_free (clock->entries);
-  clock->entries = NULL;
   GST_CLOCK_BROADCAST (clock);
   gst_system_clock_add_wakeup (sysclock);
   GST_OBJECT_UNLOCK (clock);
@@ -211,6 +208,10 @@ gst_system_clock_dispose (GObject * object)
   sysclock->thread = NULL;
   GST_CAT_DEBUG (GST_CAT_CLOCK, "joined thread");
 
+  g_list_foreach (clock->entries, (GFunc) gst_clock_id_unref, NULL);
+  g_list_free (clock->entries);
+  clock->entries = NULL;
+
   gst_poll_free (sysclock->priv->timer);
 
   G_OBJECT_CLASS (parent_class)->dispose (object);
