@@ -28,8 +28,6 @@
 
 #include <string.h>
 
-/* FIXME 0.11: inline everything and get rid of non-inlined functions */
-
 /**
  * SECTION:gstbytereader
  * @short_description: Reads different integer, string and floating point
@@ -181,9 +179,7 @@ gst_byte_reader_set_pos (GstByteReader * reader, guint pos)
 guint
 gst_byte_reader_get_pos (const GstByteReader * reader)
 {
-  g_return_val_if_fail (reader != NULL, 0);
-
-  return reader->byte;
+  return _gst_byte_reader_get_pos_inline (reader);
 }
 
 /**
@@ -199,12 +195,27 @@ gst_byte_reader_get_pos (const GstByteReader * reader)
 guint
 gst_byte_reader_get_remaining (const GstByteReader * reader)
 {
-  g_return_val_if_fail (reader != NULL, 0);
-
   return _gst_byte_reader_get_remaining_inline (reader);
 }
 
+/**
+ * gst_byte_reader_get_size:
+ * @reader: a #GstByteReader instance
+ *
+ * Returns the total number of bytes of a #GstByteReader instance.
+ *
+ * Returns: The total number of bytes of @reader instance.
+ * 
+ * Since: 0.10.26
+ */
+guint
+gst_byte_reader_get_size (const GstByteReader * reader)
+{
+  return _gst_byte_reader_get_size_inline (reader);
+}
+
 #define gst_byte_reader_get_remaining _gst_byte_reader_get_remaining_inline
+#define gst_byte_reader_get_size _gst_byte_reader_get_size_inline
 
 /**
  * gst_byte_reader_skip:
@@ -220,14 +231,7 @@ gst_byte_reader_get_remaining (const GstByteReader * reader)
 gboolean
 gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
 {
-  g_return_val_if_fail (reader != NULL, FALSE);
-
-  if (gst_byte_reader_get_remaining (reader) < nbytes)
-    return FALSE;
-
-  reader->byte += nbytes;
-
-  return TRUE;
+  return _gst_byte_reader_skip_inline (reader, nbytes);
 }
 
 /**
@@ -259,7 +263,7 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
  * @reader: a #GstByteReader instance
  * @val: Pointer to a #guint8 to store the result
  *
- * Read a signed 8 bit integer into @val but keep the current position.
+ * Read an unsigned 8 bit integer into @val but keep the current position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
  * 
@@ -309,7 +313,7 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
  * @reader: a #GstByteReader instance
  * @val: Pointer to a #guint16 to store the result
  *
- * Read a signed 16 bit little endian integer into @val
+ * Read an unsigned 16 bit little endian integer into @val
  * but keep the current position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
@@ -361,7 +365,7 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
  * @reader: a #GstByteReader instance
  * @val: Pointer to a #guint16 to store the result
  *
- * Read a signed 16 bit big endian integer into @val
+ * Read an unsigned 16 bit big endian integer into @val
  * but keep the current position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
@@ -413,7 +417,7 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
  * @reader: a #GstByteReader instance
  * @val: Pointer to a #guint32 to store the result
  *
- * Read a signed 24 bit little endian integer into @val
+ * Read an unsigned 24 bit little endian integer into @val
  * but keep the current position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
@@ -465,7 +469,7 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
  * @reader: a #GstByteReader instance
  * @val: Pointer to a #guint32 to store the result
  *
- * Read a signed 24 bit big endian integer into @val
+ * Read an unsigned 24 bit big endian integer into @val
  * but keep the current position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
@@ -518,7 +522,7 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
  * @reader: a #GstByteReader instance
  * @val: Pointer to a #guint32 to store the result
  *
- * Read a signed 32 bit little endian integer into @val
+ * Read an unsigned 32 bit little endian integer into @val
  * but keep the current position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
@@ -570,7 +574,7 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
  * @reader: a #GstByteReader instance
  * @val: Pointer to a #guint32 to store the result
  *
- * Read a signed 32 bit big endian integer into @val
+ * Read an unsigned 32 bit big endian integer into @val
  * but keep the current position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
@@ -622,7 +626,7 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
  * @reader: a #GstByteReader instance
  * @val: Pointer to a #guint64 to store the result
  *
- * Read a signed 64 bit little endian integer into @val
+ * Read an unsigned 64 bit little endian integer into @val
  * but keep the current position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
@@ -674,7 +678,7 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
  * @reader: a #GstByteReader instance
  * @val: Pointer to a #guint64 to store the result
  *
- * Read a signed 64 bit big endian integer into @val
+ * Read an unsigned 64 bit big endian integer into @val
  * but keep the current position.
  *
  * Returns: %TRUE if successful, %FALSE otherwise.
@@ -699,18 +703,12 @@ gst_byte_reader_skip (GstByteReader * reader, guint nbytes)
 gboolean \
 gst_byte_reader_get_##name (GstByteReader * reader, type * val) \
 { \
-  g_return_val_if_fail (reader != NULL, FALSE); \
-  g_return_val_if_fail (val != NULL, FALSE); \
-  \
   return _gst_byte_reader_get_##name##_inline (reader, val); \
 } \
 \
 gboolean \
 gst_byte_reader_peek_##name (GstByteReader * reader, type * val) \
 { \
-  g_return_val_if_fail (reader != NULL, FALSE); \
-  g_return_val_if_fail (val != NULL, FALSE); \
-  \
   return _gst_byte_reader_peek_##name##_inline (reader, val); \
 }
 
@@ -869,9 +867,6 @@ gboolean
 gst_byte_reader_get_data (GstByteReader * reader, guint size,
     const guint8 ** val)
 {
-  g_return_val_if_fail (reader != NULL, FALSE);
-  g_return_val_if_fail (val != NULL, FALSE);
-
   return _gst_byte_reader_get_data_inline (reader, size, val);
 }
 
@@ -894,9 +889,6 @@ gboolean
 gst_byte_reader_peek_data (GstByteReader * reader, guint size,
     const guint8 ** val)
 {
-  g_return_val_if_fail (reader != NULL, FALSE);
-  g_return_val_if_fail (val != NULL, FALSE);
-
   return _gst_byte_reader_peek_data_inline (reader, size, val);
 }
 
@@ -917,9 +909,6 @@ gst_byte_reader_peek_data (GstByteReader * reader, guint size,
 gboolean
 gst_byte_reader_dup_data (GstByteReader * reader, guint size, guint8 ** val)
 {
-  g_return_val_if_fail (reader != NULL, FALSE);
-  g_return_val_if_fail (val != NULL, FALSE);
-
   return _gst_byte_reader_dup_data_inline (reader, size, val);
 }
 
