@@ -29,6 +29,7 @@
  */
 
 #include <gst/gst.h>
+#include "string.h"
 #include "gstdataqueue.h"
 
 GST_DEBUG_CATEGORY_STATIC (data_queue_debug);
@@ -195,6 +196,8 @@ gst_data_queue_init (GstDataQueue * queue)
  * or @emptycallback.
  *
  * Returns: a new #GstDataQueue.
+ *
+ * Since: 0.10.26
  */
 
 GstDataQueue *
@@ -216,11 +219,17 @@ gst_data_queue_new_full (GstDataQueueCheckFullFunction checkfull,
 }
 
 /**
- * gst_data_queue_new:
+ * gst_data_queue_new_full:
  * @checkfull: the callback used to tell if the element considers the queue full
  * or not.
+ * @fullcallback: the callback which will be called when the queue is considered full.
+ * @emptycallback: the callback which will be called when the queue is considered empty.
  * @checkdata: a #gpointer that will be given in the @checkfull callback.
  *
+ * Creates a new #GstDataQueue. The difference with @gst_data_queue_new is that it will
+ * not emit the 'full' and 'empty' signals, but instead calling directly @fullcallback
+ * or @emptycallback.
+ *
  * Returns: a new #GstDataQueue.
  */
 
@@ -265,7 +274,7 @@ gst_data_queue_finalize (GObject * object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
-static void
+static inline void
 gst_data_queue_locked_flush (GstDataQueue * queue)
 {
   STATUS (queue, "before flushing");
@@ -275,13 +284,13 @@ gst_data_queue_locked_flush (GstDataQueue * queue)
   g_cond_signal (queue->item_del);
 }
 
-static gboolean
+static inline gboolean
 gst_data_queue_locked_is_empty (GstDataQueue * queue)
 {
   return (queue->queue->length == 0);
 }
 
-static gboolean
+static inline gboolean
 gst_data_queue_locked_is_full (GstDataQueue * queue)
 {
   return queue->checkfull (queue, queue->cur_level.visible,
@@ -585,9 +594,7 @@ gst_data_queue_limits_changed (GstDataQueue * queue)
 void
 gst_data_queue_get_level (GstDataQueue * queue, GstDataQueueSize * level)
 {
-  level->visible = queue->cur_level.visible;
-  level->bytes = queue->cur_level.bytes;
-  level->time = queue->cur_level.time;
+  memcpy (level, (&queue->cur_level), sizeof (GstDataQueueSize));
 }
 
 static void
